import React from 'react';
import PropTypes from 'prop-types';
import { Dialog } from 'primereact/dialog';
import ActionButton from './ActionButton';

export const YesNoDialog = props => {
	const {
		children,
		className,
		header,
		id,
		key,
		name,
		noButtonClassName,
		noButtonLabel,
		noButtonVariant,
		onAfterNoStateChange,
		onAfterYesStateChange,
		onChange,
		stateField,
		visible,
		yesButtonClassName,
		yesButtonLabel,
		yesButtonVariant,
	} = props;
	return (
		<Dialog
			ariaCloseIconLabel="Zamknij okno dialogowe"
			header={header}
			visible={visible}
			className={className}
			style={{ width: '30vw' }}
			key={`${key ? key : id ? id : name}-dialog`}
			onHide={() =>
				onChange
					? onChange(
							'YES_NO_DIALOG',
							undefined,
							{ name, value: false },
							onAfterNoStateChange,
							stateField
					  )
					: null
			}
		>
			<div style={{ marginBottom: '20px' }} className="row col-12">
				{children ? children : 'Czy jesteś pewien?'}
			</div>
			<div className="row col-12">
				<ActionButton
					label={yesButtonLabel}
					variant={yesButtonVariant}
					className={yesButtonClassName}
					handleClick={() =>
						onChange
							? onChange(
									'YES_NO_DIALOG',
									undefined,
									{ name, value: true },
									onAfterYesStateChange,
									stateField
							  )
							: null
					}
					key={`${key ? key : id ? id : name}-yes-button`}
				/>
				<ActionButton
					label={noButtonLabel}
					variant={noButtonVariant}
					className={noButtonClassName}
					handleClick={() =>
						onChange
							? onChange(
									'YES_NO_DIALOG',
									undefined,
									{ name, value: false },
									onAfterNoStateChange,
									stateField
							  )
							: null
					}
					key={`${key ? key : id ? id : name}-no-button`}
				/>
			</div>
		</Dialog>
	);
};

YesNoDialog.defaultProps = {
	noButtonVariant: 'accent',
	noButtonLabel: 'Nie',
	stateField: 'element',
	yesButtonVariant: 'dark',
	yesButtonLabel: 'Tak',
};

YesNoDialog.propTypes = {
	children: PropTypes.oneOfType([PropTypes.arrayOf(PropTypes.node), PropTypes.node]),
	className: PropTypes.string,
	header: PropTypes.string.isRequired,
	id: PropTypes.string,
	key: PropTypes.string,
	name: PropTypes.string.isRequired,
	noButtonClassName: PropTypes.string,
	noButtonLabel: PropTypes.string,
	noButtonVariant: PropTypes.string,
	onAfterNoStateChange: PropTypes.func,
	onAfterYesStateChange: PropTypes.func,
	onChange: PropTypes.func.isRequired,
	stateField: PropTypes.string,
	visible: PropTypes.bool.isRequired,
	yesButtonClassName: PropTypes.string,
	yesButtonLabel: PropTypes.string,
	yesButtonVariant: PropTypes.string,
};

export default YesNoDialog;
